/**************************************************************************
 * 
 * RGB clock to teach kids about time 
 * With 
 * NTP & DST
 * ESP8266 Wemos d1 mini board
 * ST7735 TFT display.
 * Created by:
 * Danniel
 * https://www.instructables.com/member/dannield/instructables/
 * 
 * Created: 10/10/2020 
 *************************************************************************/
 
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <NTPClient.h>        // include NTPClient library
#include <TimeLib.h>          // Include Arduino time library
#include <Adafruit_GFX.h>     // include Adafruit graphics library
#include <Adafruit_ST7735.h>  // include Adafruit ST7735 TFT library
#include <Timezone.h>         // include Timezone library

// Israel Time Zone (IST/IDT)
TimeChangeRule myDST = {"IDT", Last, Sun, Mar, 2, 60};    // Daylight time = UTC + 3 hours
TimeChangeRule mySTD = {"IST", Last, Sun, Oct, 2, 0};     // Standard time = UTC + 2 hours
Timezone myTZ(myDST, mySTD);
TimeChangeRule *tcr;

//24 hours collor pallet and B/W text color
int HourColor[24] = {0x0000, 0x0000, 0x0000, 0x0000, 0x1808, 0x098F, 0x0A54, 0x1B79, 0x3D1E, 0x869F, 0xA79F, 0xCFDF, 0xFFDB, 0xFF4F, 0xFE07, 0xFD00, 0xFB00, 0xF980, 0xA987, 0x618D, 0x1808, 0x0000, 0x0000, 0x0000};
int ClockColor[24] = {0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF};

unsigned int FGColor = 0xFFFF;
unsigned int BGColor = 0x0000;
time_t utc;
time_t local;

// ST7735 TFT module connections
#define TFT_RST   D4     // TFT RST pin is connected to WeMos pin D4 (GPIO2)
#define TFT_CS    D3     // TFT CS  pin is connected to WeMos pin D3 (GPIO0)
#define TFT_DC    D2     // TFT DC  pin is connected to WeMos pin D2 (GPIO4)
// initialize ST7735 TFT library with hardware SPI module
// SCK (CLK) ---> WeMos pin D5 (GPIO14)
// MOSI(DIN) ---> WeMos pin D7 (GPIO13)
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);
 
// set WiFi network SSID and password
const char *ssid     = "SSID";
const char *password = "PASSWORD";
 
WiFiUDP ntpUDP;
 
// 'time.nist.gov' is used (default server) with +1 hour offset (3600 seconds) 60 seconds (60000 milliseconds) update interval
NTPClient timeClient(ntpUDP, "0.pool.ntp.org", 7200, 60000);
unsigned long unix_epoch;

void paintGradient(){
  int gradTop = 0;
  int gradHight = 35;
  int gradStart = 0;
  int GradStep = 5;

  for (int i = 0; i < 24; i++) {
    tft.fillRect(gradStart+(GradStep*i), gradTop, GradStep+(GradStep*i), gradHight, HourColor[i]);
  }
}
 
void setup(void){
  Serial.begin(9600);
  tft.initR(INITR_BLACKTAB);     // initialize a ST7735S chip, black tab
  tft.fillScreen(BGColor);  // fill screen with black color
  tft.drawFastHLine(0, 44,  tft.width(), ST7735_BLUE);    // draw horizontal blue line at position (0, 44)
  tft.setTextColor(ST7735_MAGENTA, BGColor);   // set text color to white and black background
  tft.setTextSize(1);                 // text size = 1
  tft.setCursor(21, 10);              // move cursor to position (43, 10) pixel
  tft.print("Color Clock");
  paintGradient();
  
 
  WiFi.begin(ssid, password);
  Serial.print("Connecting.");
  tft.setCursor(0, 54);              // move cursor to position (4, 27) pixel
  tft.print("Connecting..");
  while ( WiFi.status() != WL_CONNECTED ) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("connected");
  tft.print("connected");
  delay(2000);
  
  tft.fillRect(0, 54, tft.width(), 8, ST7735_BLACK);
 
  tft.drawFastHLine(0, 102, tft.width(), ST7735_BLUE);  // draw horizontal blue line at position (0, 102)
  tft.setTextSize(2);                                   // text size = 2
  tft.setTextColor(ST7735_MAGENTA, ST7735_BLACK);       // set text color to magenta and black background
  tft.setCursor(37, 112);                               // move cursor to position (37, 112) pixel
  tft.print("TIME:");
 
  timeClient.begin();

  timeUpdate();
  staticDraw();
}
 
void RTC_display(){
  if ( minute(local) == 0 && second(local) == 0) {
    staticDraw();
  }

  // print time
  tft.setCursor(16, 136);
  tft.setTextColor(FGColor, BGColor);     // set text color to green and black background
  tft.printf( "%02u:%02u:%02u", hour(local), minute(local), second(local) );
}
 
// main loop
void loop(){
  timeUpdate();
  RTC_display();
  delay(500);    // wait 200ms
 
}

void staticDraw() {
    tft.fillScreen(BGColor);  // fill screen with black color
    paintGradient();
   
    tft.setTextSize(2);                                   // text size = 2
    tft.setTextColor(ST7735_MAGENTA, BGColor);       // set text color to magenta and black background
    tft.setCursor(37, 112);                               // move cursor to position (37, 112) pixel
    tft.print("TIME:");
    char dow_matrix[7][10] = {"SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY"};
    byte x_pos[7] = {29, 29, 23, 11, 17, 29, 17};
    static byte previous_dow = 0;
    
    previous_dow = weekday(unix_epoch);
    tft.fillRect(11, 55, 108, 14, BGColor);     // draw rectangle (erase day from the display)
    tft.setCursor(x_pos[previous_dow-1], 55);
    tft.setTextColor(ST7735_MAGENTA, BGColor);     // set text color to cyan and black background
    tft.print( dow_matrix[previous_dow-1] );
   
    // print date
    tft.setCursor(4, 79);
    tft.setTextColor(FGColor, BGColor);     // set text color to yellow and black background
    tft.printf( "%02u-%02u-%04u", day(unix_epoch), month(unix_epoch), year(unix_epoch) );
}

void timeUpdate(){
  timeClient.update();
  unix_epoch = timeClient.getEpochTime();   // get UNIX Epoch time
  utc = unix_epoch;
  local = myTZ.toLocal(utc, &tcr);
  BGColor = HourColor[hour(local)];
  FGColor = ClockColor[hour(local)];
}
